@@ -6,9 +6,11 @@ import (
 	"testing"
 	"time"
 
+	ign3types "github.com/coreos/ignition/v2/config/v3_2/types"
 	"github.com/golang/glog"
-
-	v1 "k8s.io/api/core/v1"
+	osev1 "github.com/openshift/api/config/v1"
+	oseconfigfake "github.com/openshift/client-go/config/clientset/versioned/fake"
+	oseinformersv1 "github.com/openshift/client-go/config/informers/externalversions"
 	"k8s.io/apimachinery/pkg/api/equality"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
@@ -21,12 +23,8 @@ import (
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/tools/record"
 	kubeletconfigv1beta1 "k8s.io/kubelet/config/v1beta1"
+	"k8s.io/utils/pointer"
 
-	osev1 "github.com/openshift/api/config/v1"
-	oseinformersv1 "github.com/openshift/client-go/config/informers/externalversions"
-
-	igntypes "github.com/coreos/ignition/config/v2_2/types"
-	oseconfigfake "github.com/openshift/client-go/config/clientset/versioned/fake"
 	mcfgv1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
 	ctrlcommon "github.com/openshift/machine-config-operator/pkg/controller/common"
 	"github.com/openshift/machine-config-operator/pkg/generated/clientset/versioned/fake"
@@ -97,20 +95,28 @@ func newFeatures(name string, enabled, disabled []string, labels map[string]stri
 	}
 }
 
-func newControllerConfig(name, platform string) *mcfgv1.ControllerConfig {
+func newControllerConfig(name string, platform osev1.PlatformType) *mcfgv1.ControllerConfig {
 	cc := &mcfgv1.ControllerConfig{
 		TypeMeta:   metav1.TypeMeta{APIVersion: mcfgv1.SchemeGroupVersion.String()},
 		ObjectMeta: metav1.ObjectMeta{Name: name, UID: types.UID(utilrand.String(5))},
 		Spec: mcfgv1.ControllerConfigSpec{
-			EtcdDiscoveryDomain: fmt.Sprintf("%s.tt.testing", name),
-			Platform:            platform,
+			Infra: &osev1.Infrastructure{
+				Status: osev1.InfrastructureStatus{
+					EtcdDiscoveryDomain: fmt.Sprintf("%s.tt.testing", name),
+					PlatformStatus: &osev1.PlatformStatus{
+						Type: platform,
+					},
+					APIServerURL:         fmt.Sprintf("https://api.%s.tt.testing:6443", name),
+					APIServerInternalURL: fmt.Sprintf("https://api-int.%s.tt.testing:6443", name),
+				},
+			},
 		},
 	}
 	return cc
 }
 
 func newKubeletConfig(name string, kubeconf *kubeletconfigv1beta1.KubeletConfiguration, selector *metav1.LabelSelector) *mcfgv1.KubeletConfig {
-	kcRaw, err := encodeKubeletConfig(kubeconf, kubeletconfigv1beta1.SchemeGroupVersion)
+	kcRaw, err := EncodeKubeletConfig(kubeconf, kubeletconfigv1beta1.SchemeGroupVersion)
 	if err != nil {
 		panic(err)
 	}
@@ -119,6 +125,7 @@ func newKubeletConfig(name string, kubeconf *kubeletconfigv1beta1.KubeletConfigu
 		TypeMeta:   metav1.TypeMeta{APIVersion: mcfgv1.SchemeGroupVersion.String()},
 		ObjectMeta: metav1.ObjectMeta{Name: name, UID: types.UID(utilrand.String(5)), Generation: 1},
 		Spec: mcfgv1.KubeletConfigSpec{
+			LogLevel: pointer.Int32Ptr(2),
 			KubeletConfig: &runtime.RawExtension{
 				Raw: kcRaw,
 			},
@@ -149,11 +156,6 @@ func (f *fixture) newController() *Controller {
 	c.featListerSynced = alwaysReady
 	c.eventRecorder = &record.FakeRecorder{}
 
-	c.patchKubeletConfigsFunc = func(name string, patch []byte) error {
-		f.client.Invokes(core.NewRootPatchAction(schema.GroupVersionResource{Version: "v1", Group: "machineconfiguration.openshift.io", Resource: "kubeletconfigs"}, name, types.MergePatchType, patch), nil)
-		return nil
-	}
-
 	stopCh := make(chan struct{})
 	defer close(stopCh)
 	i.Start(stopCh)
@@ -313,16 +315,19 @@ func (f *fixture) expectUpdateKubeletConfig(config *mcfgv1.KubeletConfig) {
 }
 
 func TestKubeletConfigCreate(t *testing.T) {
-	for _, platform := range []string{"aws", "none", "unrecognized"} {
-		t.Run(platform, func(t *testing.T) {
+	for _, platform := range []osev1.PlatformType{osev1.AWSPlatformType, osev1.NonePlatformType, "unrecognized"} {
+		t.Run(string(platform), func(t *testing.T) {
 			f := newFixture(t)
+			f.newController()
 
 			cc := newControllerConfig(ctrlcommon.ControllerConfigName, platform)
 			mcp := helpers.NewMachineConfigPool("master", nil, helpers.MasterSelector, "v0")
-			mcp.ObjectMeta.Labels["kubeletType"] = "small-pods"
 			mcp2 := helpers.NewMachineConfigPool("worker", nil, helpers.WorkerSelector, "v0")
-			kc1 := newKubeletConfig("smaller-max-pods", &kubeletconfigv1beta1.KubeletConfiguration{MaxPods: 100}, metav1.AddLabelToSelector(&metav1.LabelSelector{}, "kubeletType", "small-pods"))
-			mcs := helpers.NewMachineConfig(getManagedKubeletConfigKey(mcp), map[string]string{"node-role/master": ""}, "dummy://", []igntypes.File{{}})
+			kc1 := newKubeletConfig("smaller-max-pods", &kubeletconfigv1beta1.KubeletConfiguration{MaxPods: 100}, metav1.AddLabelToSelector(&metav1.LabelSelector{}, "pools.operator.machineconfiguration.openshift.io/master", ""))
+			kubeletConfigKey, _ := getManagedKubeletConfigKey(mcp, f.client, kc1)
+			mcs := helpers.NewMachineConfig(kubeletConfigKey, map[string]string{"node-role/master": ""}, "dummy://", []ign3types.File{{}})
+			mcsDeprecated := mcs.DeepCopy()
+			mcsDeprecated.Name = getManagedKubeletConfigKeyDeprecated(mcp)
 
 			f.ccLister = append(f.ccLister, cc)
 			f.mcpLister = append(f.mcpLister, mcp)
@@ -330,6 +335,49 @@ func TestKubeletConfigCreate(t *testing.T) {
 			f.mckLister = append(f.mckLister, kc1)
 			f.objects = append(f.objects, kc1)
 
+			f.expectGetMachineConfigAction(mcs)
+			f.expectGetMachineConfigAction(mcsDeprecated)
+			f.expectGetMachineConfigAction(mcs)
+			f.expectCreateMachineConfigAction(mcs)
+			f.expectPatchKubeletConfig(kc1, []uint8{0x7b, 0x22, 0x6d, 0x65, 0x74, 0x61, 0x64, 0x61, 0x74, 0x61, 0x22, 0x3a, 0x7b, 0x22, 0x66, 0x69, 0x6e, 0x61, 0x6c, 0x69, 0x7a, 0x65, 0x72, 0x73, 0x22, 0x3a, 0x5b, 0x22, 0x39, 0x39, 0x2d, 0x6d, 0x61, 0x73, 0x74, 0x65, 0x72, 0x2d, 0x68, 0x35, 0x35, 0x32, 0x6d, 0x2d, 0x73, 0x6d, 0x61, 0x6c, 0x6c, 0x65, 0x72, 0x2d, 0x6d, 0x61, 0x78, 0x2d, 0x70, 0x6f, 0x64, 0x73, 0x2d, 0x6b, 0x75, 0x62, 0x65, 0x6c, 0x65, 0x74, 0x22, 0x5d, 0x7d, 0x7d})
+			f.expectUpdateKubeletConfig(kc1)
+
+			f.run(getKey(kc1, t))
+		})
+	}
+}
+
+func TestKubeletConfigLogFile(t *testing.T) {
+	for _, platform := range []osev1.PlatformType{osev1.AWSPlatformType, osev1.NonePlatformType, "unrecognized"} {
+		t.Run(string(platform), func(t *testing.T) {
+			f := newFixture(t)
+			f.newController()
+
+			cc := newControllerConfig(ctrlcommon.ControllerConfigName, platform)
+			mcp := helpers.NewMachineConfigPool("master", nil, helpers.MasterSelector, "v0")
+			mcp2 := helpers.NewMachineConfigPool("worker", nil, helpers.WorkerSelector, "v0")
+			kc1 := &mcfgv1.KubeletConfig{
+				TypeMeta:   metav1.TypeMeta{APIVersion: mcfgv1.SchemeGroupVersion.String()},
+				ObjectMeta: metav1.ObjectMeta{Name: "kubulet-log", UID: types.UID(utilrand.String(5)), Generation: 1},
+				Spec: mcfgv1.KubeletConfigSpec{
+					LogLevel:                  pointer.Int32Ptr(5),
+					MachineConfigPoolSelector: metav1.AddLabelToSelector(&metav1.LabelSelector{}, "pools.operator.machineconfiguration.openshift.io/master", ""),
+				},
+				Status: mcfgv1.KubeletConfigStatus{},
+			}
+			kubeletConfigKey, _ := getManagedKubeletConfigKey(mcp, f.client, kc1)
+			mcs := helpers.NewMachineConfig(kubeletConfigKey, map[string]string{"node-role/master": ""}, "dummy://", []ign3types.File{{}})
+			mcsDeprecated := mcs.DeepCopy()
+			mcsDeprecated.Name = getManagedKubeletConfigKeyDeprecated(mcp)
+
+			f.ccLister = append(f.ccLister, cc)
+			f.mcpLister = append(f.mcpLister, mcp)
+			f.mcpLister = append(f.mcpLister, mcp2)
+			f.mckLister = append(f.mckLister, kc1)
+			f.objects = append(f.objects, kc1)
+
+			f.expectGetMachineConfigAction(mcs)
+			f.expectGetMachineConfigAction(mcsDeprecated)
 			f.expectGetMachineConfigAction(mcs)
 			f.expectCreateMachineConfigAction(mcs)
 			f.expectPatchKubeletConfig(kc1, []uint8{0x7b, 0x22, 0x6d, 0x65, 0x74, 0x61, 0x64, 0x61, 0x74, 0x61, 0x22, 0x3a, 0x7b, 0x22, 0x66, 0x69, 0x6e, 0x61, 0x6c, 0x69, 0x7a, 0x65, 0x72, 0x73, 0x22, 0x3a, 0x5b, 0x22, 0x39, 0x39, 0x2d, 0x6d, 0x61, 0x73, 0x74, 0x65, 0x72, 0x2d, 0x68, 0x35, 0x35, 0x32, 0x6d, 0x2d, 0x73, 0x6d, 0x61, 0x6c, 0x6c, 0x65, 0x72, 0x2d, 0x6d, 0x61, 0x78, 0x2d, 0x70, 0x6f, 0x64, 0x73, 0x2d, 0x6b, 0x75, 0x62, 0x65, 0x6c, 0x65, 0x74, 0x22, 0x5d, 0x7d, 0x7d})
@@ -341,16 +389,19 @@ func TestKubeletConfigCreate(t *testing.T) {
 }
 
 func TestKubeletConfigUpdates(t *testing.T) {
-	for _, platform := range []string{"aws", "none", "unrecognized"} {
-		t.Run(platform, func(t *testing.T) {
+	for _, platform := range []osev1.PlatformType{osev1.AWSPlatformType, osev1.NonePlatformType, "unrecognized"} {
+		t.Run(string(platform), func(t *testing.T) {
 			f := newFixture(t)
+			f.newController()
 
 			cc := newControllerConfig(ctrlcommon.ControllerConfigName, platform)
 			mcp := helpers.NewMachineConfigPool("master", nil, helpers.MasterSelector, "v0")
-			mcp.ObjectMeta.Labels["kubeletType"] = "small-pods"
 			mcp2 := helpers.NewMachineConfigPool("worker", nil, helpers.WorkerSelector, "v0")
-			kc1 := newKubeletConfig("smaller-max-pods", &kubeletconfigv1beta1.KubeletConfiguration{MaxPods: 100}, metav1.AddLabelToSelector(&metav1.LabelSelector{}, "kubeletType", "small-pods"))
-			mcs := helpers.NewMachineConfig(getManagedKubeletConfigKey(mcp), map[string]string{"node-role/master": ""}, "dummy://", []igntypes.File{{}})
+			kc1 := newKubeletConfig("smaller-max-pods", &kubeletconfigv1beta1.KubeletConfiguration{MaxPods: 100}, metav1.AddLabelToSelector(&metav1.LabelSelector{}, "pools.operator.machineconfiguration.openshift.io/master", ""))
+			kubeletConfigKey, _ := getManagedKubeletConfigKey(mcp, f.client, kc1)
+			mcs := helpers.NewMachineConfig(kubeletConfigKey, map[string]string{"node-role/master": ""}, "dummy://", []ign3types.File{{}})
+			mcsDeprecated := mcs.DeepCopy()
+			mcsDeprecated.Name = getManagedKubeletConfigKeyDeprecated(mcp)
 
 			f.ccLister = append(f.ccLister, cc)
 			f.mcpLister = append(f.mcpLister, mcp)
@@ -358,6 +409,8 @@ func TestKubeletConfigUpdates(t *testing.T) {
 			f.mckLister = append(f.mckLister, kc1)
 			f.objects = append(f.objects, kc1)
 
+			f.expectGetMachineConfigAction(mcs)
+			f.expectGetMachineConfigAction(mcsDeprecated)
 			f.expectGetMachineConfigAction(mcs)
 			f.expectCreateMachineConfigAction(mcs)
 			f.expectPatchKubeletConfig(kc1, []uint8{0x7b, 0x22, 0x6d, 0x65, 0x74, 0x61, 0x64, 0x61, 0x74, 0x61, 0x22, 0x3a, 0x7b, 0x22, 0x66, 0x69, 0x6e, 0x61, 0x6c, 0x69, 0x7a, 0x65, 0x72, 0x73, 0x22, 0x3a, 0x5b, 0x22, 0x39, 0x39, 0x2d, 0x6d, 0x61, 0x73, 0x74, 0x65, 0x72, 0x2d, 0x68, 0x35, 0x35, 0x32, 0x6d, 0x2d, 0x73, 0x6d, 0x61, 0x6c, 0x6c, 0x65, 0x72, 0x2d, 0x6d, 0x61, 0x78, 0x2d, 0x70, 0x6f, 0x64, 0x73, 0x2d, 0x6b, 0x75, 0x62, 0x65, 0x6c, 0x65, 0x74, 0x22, 0x5d, 0x7d, 0x7d})
@@ -384,7 +437,7 @@ func TestKubeletConfigUpdates(t *testing.T) {
 				t.Errorf("KubeletConfig could not be unmarshalled")
 			}
 			kcDecoded.MaxPods = 101
-			kcRaw, err := encodeKubeletConfig(kcDecoded, kubeletconfigv1beta1.SchemeGroupVersion)
+			kcRaw, err := EncodeKubeletConfig(kcDecoded, kubeletconfigv1beta1.SchemeGroupVersion)
 			if err != nil {
 				t.Errorf("KubeletConfig could not be marshalled")
 			}
@@ -407,6 +460,7 @@ func TestKubeletConfigUpdates(t *testing.T) {
 				t.Errorf("syncHandler returned: %v", err)
 			}
 
+			f.expectGetMachineConfigAction(mcs)
 			f.expectGetMachineConfigAction(mcs)
 			f.expectUpdateMachineConfigAction(mcs)
 			f.expectPatchKubeletConfig(kcUpdate, []uint8{0x7b, 0x22, 0x6d, 0x65, 0x74, 0x61, 0x64, 0x61, 0x74, 0x61, 0x22, 0x3a, 0x7b, 0x22, 0x66, 0x69, 0x6e, 0x61, 0x6c, 0x69, 0x7a, 0x65, 0x72, 0x73, 0x22, 0x3a, 0x5b, 0x22, 0x39, 0x39, 0x2d, 0x6d, 0x61, 0x73, 0x74, 0x65, 0x72, 0x2d, 0x6d, 0x77, 0x77, 0x74, 0x67, 0x2d, 0x6b, 0x75, 0x62, 0x65, 0x6c, 0x65, 0x74, 0x22, 0x5d, 0x7d, 0x7d})
@@ -419,7 +473,7 @@ func TestKubeletConfigUpdates(t *testing.T) {
 	}
 }
 
-func TestKubeletConfigBlacklistedOptions(t *testing.T) {
+func TestKubeletConfigDenylistedOptions(t *testing.T) {
 	failureTests := []struct {
 		name   string
 		config *kubeletconfigv1beta1.KubeletConfiguration
@@ -462,121 +516,6 @@ func TestKubeletConfigBlacklistedOptions(t *testing.T) {
 				},
 			},
 		},
-		{
-			name: "evictionSoft cannot be supplied without evictionSoftGracePeriod",
-			config: &kubeletconfigv1beta1.KubeletConfiguration{
-				EvictionSoft: map[string]string{
-					"memory.available": "90%",
-				},
-			},
-		},
-		{
-			name: "evictionSoft cannot be supplied without corresponding evictionSoftGracePeriod",
-			config: &kubeletconfigv1beta1.KubeletConfiguration{
-				EvictionSoft: map[string]string{
-					"memory.available": "90%",
-				},
-				EvictionSoftGracePeriod: map[string]string{
-					"nodefs.inodesFree": "1h",
-				},
-			},
-		},
-		{
-			name: "kubeReserved cpu value cannot be negative",
-			config: &kubeletconfigv1beta1.KubeletConfiguration{
-				KubeReserved: map[string]string{
-					v1.ResourceCPU.String(): "-20",
-				},
-			},
-		},
-		{
-			name: "systemReserved cpu value cannot be negative",
-			config: &kubeletconfigv1beta1.KubeletConfiguration{
-				SystemReserved: map[string]string{
-					v1.ResourceCPU.String(): "-20",
-				},
-			},
-		},
-		{
-			name: "kubeReserved memory value cannot be negative",
-			config: &kubeletconfigv1beta1.KubeletConfiguration{
-				KubeReserved: map[string]string{
-					v1.ResourceMemory.String(): "-20M",
-				},
-			},
-		},
-		{
-			name: "systemReserved memory value cannot be negative",
-			config: &kubeletconfigv1beta1.KubeletConfiguration{
-				SystemReserved: map[string]string{
-					v1.ResourceMemory.String(): "-20M",
-				},
-			},
-		},
-		{
-			name: "kubeReserved cpu value fails to parse",
-			config: &kubeletconfigv1beta1.KubeletConfiguration{
-				KubeReserved: map[string]string{
-					v1.ResourceCPU.String(): "Peter Griffin",
-				},
-			},
-		},
-		{
-			name: "systemReserved cpu value fails to parse",
-			config: &kubeletconfigv1beta1.KubeletConfiguration{
-				SystemReserved: map[string]string{
-					v1.ResourceCPU.String(): "Stewie Griffin",
-				},
-			},
-		},
-		{
-			name: "kubeReserved memory value fails to parse",
-			config: &kubeletconfigv1beta1.KubeletConfiguration{
-				KubeReserved: map[string]string{
-					v1.ResourceMemory.String(): "Brian Griffin",
-				},
-			},
-		},
-		{
-			name: "systemReserved memory value fails to parse",
-			config: &kubeletconfigv1beta1.KubeletConfiguration{
-				SystemReserved: map[string]string{
-					v1.ResourceMemory.String(): "Meg Griffin",
-				},
-			},
-		},
-		{
-			name: "kubeReserved ephemeral-storage value fails to parse",
-			config: &kubeletconfigv1beta1.KubeletConfiguration{
-				KubeReserved: map[string]string{
-					v1.ResourceEphemeralStorage.String(): "Lois Griffin",
-				},
-			},
-		},
-		{
-			name: "kubeReserved ephemeral-storage value cannot be negative",
-			config: &kubeletconfigv1beta1.KubeletConfiguration{
-				KubeReserved: map[string]string{
-					v1.ResourceEphemeralStorage.String(): "-20M",
-				},
-			},
-		},
-		{
-			name: "systemReserved ephemeral-storage value fails to parse",
-			config: &kubeletconfigv1beta1.KubeletConfiguration{
-				SystemReserved: map[string]string{
-					v1.ResourceEphemeralStorage.String(): "Glenn Quagmire",
-				},
-			},
-		},
-		{
-			name: "systemReserved ephemeral-storage value cannot be negative",
-			config: &kubeletconfigv1beta1.KubeletConfiguration{
-				SystemReserved: map[string]string{
-					v1.ResourceEphemeralStorage.String(): "-20M",
-				},
-			},
-		},
 	}
 
 	successTests := []struct {
@@ -611,16 +550,19 @@ func TestKubeletConfigBlacklistedOptions(t *testing.T) {
 }
 
 func TestKubeletFeatureExists(t *testing.T) {
-	for _, platform := range []string{"aws", "none", "unrecognized"} {
-		t.Run(platform, func(t *testing.T) {
+	for _, platform := range []osev1.PlatformType{osev1.AWSPlatformType, osev1.NonePlatformType, "Unrecognized"} {
+		t.Run(string(platform), func(t *testing.T) {
 			f := newFixture(t)
+			f.newController()
 
 			cc := newControllerConfig(ctrlcommon.ControllerConfigName, platform)
 			mcp := helpers.NewMachineConfigPool("master", nil, helpers.MasterSelector, "v0")
-			mcp.ObjectMeta.Labels["kubeletType"] = "small-pods"
 			mcp2 := helpers.NewMachineConfigPool("worker", nil, helpers.WorkerSelector, "v0")
-			kc1 := newKubeletConfig("smaller-max-pods", &kubeletconfigv1beta1.KubeletConfiguration{MaxPods: 100}, metav1.AddLabelToSelector(&metav1.LabelSelector{}, "kubeletType", "small-pods"))
-			mcs := helpers.NewMachineConfig(getManagedKubeletConfigKey(mcp), map[string]string{"node-role/master": ""}, "dummy://", []igntypes.File{{}})
+			kc1 := newKubeletConfig("smaller-max-pods", &kubeletconfigv1beta1.KubeletConfiguration{MaxPods: 100}, metav1.AddLabelToSelector(&metav1.LabelSelector{}, "pools.operator.machineconfiguration.openshift.io/master", ""))
+			kubeletConfigKey, _ := getManagedKubeletConfigKey(mcp, f.client, kc1)
+			mcs := helpers.NewMachineConfig(kubeletConfigKey, map[string]string{"node-role/master": ""}, "dummy://", []ign3types.File{{}})
+			mcsDeprecated := mcs.DeepCopy()
+			mcsDeprecated.Name = getManagedKubeletConfigKeyDeprecated(mcp)
 
 			f.ccLister = append(f.ccLister, cc)
 			f.mcpLister = append(f.mcpLister, mcp)
@@ -631,6 +573,8 @@ func TestKubeletFeatureExists(t *testing.T) {
 			features := newFeatures("cluster", []string{"DynamicAuditing"}, []string{"ExpandPersistentVolumes"}, nil)
 			f.featLister = append(f.featLister, features)
 
+			f.expectGetMachineConfigAction(mcs)
+			f.expectGetMachineConfigAction(mcsDeprecated)
 			f.expectGetMachineConfigAction(mcs)
 			f.expectCreateMachineConfigAction(mcs)
 			f.expectPatchKubeletConfig(kc1, []uint8{0x7b, 0x22, 0x6d, 0x65, 0x74, 0x61, 0x64, 0x61, 0x74, 0x61, 0x22, 0x3a, 0x7b, 0x22, 0x66, 0x69, 0x6e, 0x61, 0x6c, 0x69, 0x7a, 0x65, 0x72, 0x73, 0x22, 0x3a, 0x5b, 0x22, 0x39, 0x39, 0x2d, 0x6d, 0x61, 0x73, 0x74, 0x65, 0x72, 0x2d, 0x68, 0x35, 0x35, 0x32, 0x6d, 0x2d, 0x73, 0x6d, 0x61, 0x6c, 0x6c, 0x65, 0x72, 0x2d, 0x6d, 0x61, 0x78, 0x2d, 0x70, 0x6f, 0x64, 0x73, 0x2d, 0x6b, 0x75, 0x62, 0x65, 0x6c, 0x65, 0x74, 0x22, 0x5d, 0x7d, 0x7d})
