@@ -3,11 +3,12 @@ package e2e_test
 import (
 	"context"
 	"fmt"
+	"strconv"
 	"strings"
 	"testing"
 	"time"
 
-	ign3types "github.com/coreos/ignition/v2/config/v3_1/types"
+	ign3types "github.com/coreos/ignition/v2/config/v3_2/types"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	corev1 "k8s.io/api/core/v1"
@@ -156,6 +157,12 @@ func TestKernelArguments(t *testing.T) {
 func TestKernelType(t *testing.T) {
 	cs := framework.NewClientSet("")
 
+	isOKD, err := isOKDCluster(cs)
+	require.Nil(t, err)
+	if isOKD {
+		t.Skip("skipping test on OKD")
+	}
+
 	unlabelFunc := labelRandomNodeFromPool(t, cs, "worker", "node-role.kubernetes.io/infra")
 
 	oldInfraRenderedConfig := getMcName(t, cs, "infra")
@@ -174,7 +181,7 @@ func TestKernelType(t *testing.T) {
 		},
 	}
 
-	_, err := cs.MachineConfigs().Create(context.TODO(), kernelType, metav1.CreateOptions{})
+	_, err = cs.MachineConfigs().Create(context.TODO(), kernelType, metav1.CreateOptions{})
 	require.Nil(t, err)
 	t.Logf("Created %s", kernelType.Name)
 	renderedConfig, err := waitForRenderedConfig(t, cs, "infra", kernelType.Name)
@@ -252,7 +259,7 @@ func TestExtensions(t *testing.T) {
 			Config: runtime.RawExtension{
 				Raw: helpers.MarshalOrDie(ctrlcommon.NewIgnConfig()),
 			},
-			Extensions: []string{"usbguard"},
+			Extensions: []string{"usbguard", "kernel-devel"},
 		},
 	}
 
@@ -269,10 +276,25 @@ func TestExtensions(t *testing.T) {
 	assert.Equal(t, infraNode.Annotations[constants.CurrentMachineConfigAnnotationKey], renderedConfig)
 	assert.Equal(t, infraNode.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
 
-	installedExtesnions := execCmdOnNode(t, cs, infraNode, "chroot", "/rootfs", "rpm", "-qa", "usbguard")
-	if !strings.Contains(installedExtesnions, "usbguard") {
-		t.Fatalf("Node %s doesn't have expected extensions", infraNode.Name)
+	isOKD, err := isOKDCluster(cs)
+	require.Nil(t, err)
+
+	var installedPackages string
+	var expectedPackages []string
+	if isOKD {
+		// OKD does not support grouped extensions yet, so installing kernel-devel will not also pull in kernel-headers
+		installedPackages = execCmdOnNode(t, cs, infraNode, "chroot", "/rootfs", "rpm", "-q", "usbguard", "kernel-devel")
+		expectedPackages = []string{"usbguard", "kernel-devel"}
+	} else {
+		installedPackages = execCmdOnNode(t, cs, infraNode, "chroot", "/rootfs", "rpm", "-q", "usbguard", "kernel-devel", "kernel-headers")
+		expectedPackages = []string{"usbguard", "kernel-devel", "kernel-headers"}
 	}
+	for _, v := range expectedPackages {
+		if !strings.Contains(installedPackages, v) {
+			t.Fatalf("Node %s doesn't have expected extensions", infraNode.Name)
+		}
+	}
+
 	t.Logf("Node %s has expected extensions installed", infraNode.Name)
 
 	// Delete the applied kerneltype MachineConfig to make sure rollback works fine
@@ -292,12 +314,143 @@ func TestExtensions(t *testing.T) {
 
 	assert.Equal(t, infraNode.Annotations[constants.CurrentMachineConfigAnnotationKey], oldInfraRenderedConfig)
 	assert.Equal(t, infraNode.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
-	installedExtesnions = execCmdOnNode(t, cs, infraNode, "chroot", "/rootfs", "rpm", "-qa", "usbguard")
-	if strings.Contains(installedExtesnions, "usbguard") {
+
+	if isOKD {
+		// OKD does not support grouped extensions yet, so installing kernel-devel will not also pull in kernel-headers
+		installedPackages = execCmdOnNode(t, cs, infraNode, "chroot", "/rootfs", "rpm", "-qa", "usbguard", "kernel-devel")
+	} else {
+		installedPackages = execCmdOnNode(t, cs, infraNode, "chroot", "/rootfs", "rpm", "-qa", "usbguard", "kernel-devel", "kernel-headers")
+	}
+	for _, v := range expectedPackages {
+		if strings.Contains(installedPackages, v) {
+			t.Fatalf("Node %s did not rollback successfully", infraNode.Name)
+		}
+	}
+
+	t.Logf("Node %s has successfully rolled back", infraNode.Name)
+
+	unlabelFunc()
+
+	workerMCP, err := cs.MachineConfigPools().Get(context.TODO(), "worker", metav1.GetOptions{})
+	require.Nil(t, err)
+	if err := wait.Poll(2*time.Second, 5*time.Minute, func() (bool, error) {
+		node, err := cs.Nodes().Get(context.TODO(), infraNode.Name, metav1.GetOptions{})
+		require.Nil(t, err)
+		if node.Annotations[constants.DesiredMachineConfigAnnotationKey] != workerMCP.Spec.Configuration.Name {
+			return false, nil
+		}
+		return true, nil
+	}); err != nil {
+		t.Errorf("infra node hasn't moved back to worker config: %v", err)
+	}
+	err = waitForPoolComplete(t, cs, "infra", oldInfraRenderedConfig)
+	require.Nil(t, err)
+}
+
+func TestNoReboot(t *testing.T) {
+	cs := framework.NewClientSet("")
+	unlabelFunc := labelRandomNodeFromPool(t, cs, "worker", "node-role.kubernetes.io/infra")
+	oldInfraRenderedConfig := getMcName(t, cs, "infra")
+
+	infraNode := getSingleNodeByRole(t, cs, "infra")
+
+	output := execCmdOnNode(t, cs, infraNode, "cat", "/rootfs/proc/uptime")
+	oldTime := strings.Split(output, " ")[0]
+	t.Logf("Node %s initial uptime: %s", infraNode.Name, oldTime)
+
+	// Adding authorized key for user core
+	testIgnConfig := ctrlcommon.NewIgnConfig()
+	testSSHKey := ign3types.PasswdUser{Name: "core", SSHAuthorizedKeys: []ign3types.SSHAuthorizedKey{"test adding authorized key without node reboot"}}
+	testIgnConfig.Passwd.Users = append(testIgnConfig.Passwd.Users, testSSHKey)
+
+	addAuthorizedKey := &mcfgv1.MachineConfig{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:   fmt.Sprintf("authorzied-key-infra-%s", uuid.NewUUID()),
+			Labels: mcLabelForRole("infra"),
+		},
+		Spec: mcfgv1.MachineConfigSpec{
+			Config: runtime.RawExtension{
+				Raw: helpers.MarshalOrDie(testIgnConfig),
+			},
+		},
+	}
+
+	_, err := cs.MachineConfigs().Create(context.TODO(), addAuthorizedKey, metav1.CreateOptions{})
+	require.Nil(t, err, "failed to create MC")
+	t.Logf("Created %s", addAuthorizedKey.Name)
+
+	// grab the latest worker- MC
+	renderedConfig, err := waitForRenderedConfig(t, cs, "infra", addAuthorizedKey.Name)
+	require.Nil(t, err)
+	err = waitForPoolComplete(t, cs, "infra", renderedConfig)
+	require.Nil(t, err)
+
+	// Re-fetch the infra node for updated annotations
+	infraNode = getSingleNodeByRole(t, cs, "infra")
+
+	assert.Equal(t, infraNode.Annotations[constants.CurrentMachineConfigAnnotationKey], renderedConfig)
+	assert.Equal(t, infraNode.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
+
+	foundSSHKey := execCmdOnNode(t, cs, infraNode, "cat", "/rootfs/home/core/.ssh/authorized_keys")
+	if !strings.Contains(foundSSHKey, "test adding authorized key without node reboot") {
+		t.Fatalf("updated ssh keys not found in authorized_keys, got %s", foundSSHKey)
+	}
+	t.Logf("Node %s has SSH key", infraNode.Name)
+
+	output = execCmdOnNode(t, cs, infraNode, "cat", "/rootfs/proc/uptime")
+	newTime := strings.Split(output, " ")[0]
+
+	// To ensure we didn't reboot, new uptime should be greater than old uptime
+	uptimeOld, err := strconv.ParseFloat(oldTime, 64)
+	require.Nil(t, err)
+
+	uptimeNew, err := strconv.ParseFloat(newTime, 64)
+	require.Nil(t, err)
+
+	if uptimeOld > uptimeNew {
+		t.Fatalf("Node %s rebooted uptime decreased from %f to %f", infraNode.Name, uptimeOld, uptimeNew)
+	}
+
+	t.Logf("Node %s didn't reboot as expected, uptime increased from %f to %f ", infraNode.Name, uptimeOld, uptimeNew)
+
+	// Delete the applied authorized key MachineConfig to make sure rollback works fine without node reboot
+	if err := cs.MachineConfigs().Delete(context.TODO(), addAuthorizedKey.Name, metav1.DeleteOptions{}); err != nil {
+		t.Error(err)
+	}
+
+	t.Logf("Deleted MachineConfig %s", addAuthorizedKey.Name)
+
+	// Wait for the mcp to rollback to previous config
+	if err := waitForPoolComplete(t, cs, "infra", oldInfraRenderedConfig); err != nil {
+		t.Fatal(err)
+	}
+
+	// Re-fetch the infra node for updated annotations
+	infraNode = getSingleNodeByRole(t, cs, "infra")
+
+	assert.Equal(t, infraNode.Annotations[constants.CurrentMachineConfigAnnotationKey], oldInfraRenderedConfig)
+	assert.Equal(t, infraNode.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
+
+	foundSSHKey = execCmdOnNode(t, cs, infraNode, "cat", "/rootfs/home/core/.ssh/authorized_keys")
+	if strings.Contains(foundSSHKey, "test adding authorized key without node reboot") {
 		t.Fatalf("Node %s did not rollback successfully", infraNode.Name)
 	}
+
 	t.Logf("Node %s has successfully rolled back", infraNode.Name)
 
+	// Ensure that node didn't reboot during rollback
+	output = execCmdOnNode(t, cs, infraNode, "cat", "/rootfs/proc/uptime")
+	newTime = strings.Split(output, " ")[0]
+
+	uptimeNew, err = strconv.ParseFloat(newTime, 64)
+	require.Nil(t, err)
+
+	if uptimeOld > uptimeNew {
+		t.Fatalf("Node %s rebooted during rollback, uptime decreased from %f to %f", infraNode.Name, uptimeOld, uptimeNew)
+	}
+
+	t.Logf("Node %s didn't reboot as expected during rollback, uptime increased from %f to %f ", infraNode.Name, uptimeOld, uptimeNew)
+
 	unlabelFunc()
 
 	workerMCP, err := cs.MachineConfigPools().Get(context.TODO(), "worker", metav1.GetOptions{})
@@ -540,7 +693,7 @@ func TestIgn3Cfg(t *testing.T) {
 	testIgn3Config := ign3types.Config{}
 	tempUser := ign3types.PasswdUser{Name: "core", SSHAuthorizedKeys: []ign3types.SSHAuthorizedKey{"1234_test_ign3"}}
 	testIgn3Config.Passwd.Users = append(testIgn3Config.Passwd.Users, tempUser)
-	testIgn3Config.Ignition.Version = "3.1.0"
+	testIgn3Config.Ignition.Version = "3.2.0"
 	mode := 420
 	testfiledata := "data:,test-ign3-stuff"
 	tempFile := ign3types.File{Node: ign3types.Node{Path: "/etc/testfileconfig"},
